package org.avaje.config;

import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;

/**
 * Provides access to properties loaded from the application.properties file.
 * <p>
 * Allows for loading of external properties by specifying a load.properties
 * entry in the properties file that references a file typically using an
 * environment variable.
 * </p>
 */
public final class AppProperties {

  private static PropertyMap globalMap;

  /**
   * Parse the string replacing any expressions like ${catalina.base}.
   * <p>
   * This will evaluate expressions using first environment variables, than java
   * system properties and lastly local properties - in that order.
   * </p>
   * <p>
   * Expressions start with "${" and end with "}".
   * </p>
   */
  public static String eval(String val) {
    return getPropertyMap().eval(val);
  }

  /**
   * Return as a traditional Properties object.
   */
  public static Properties asProperties() {
    return getPropertyMap().asProperties();
  }

  private static void initPropertyMap() {

    String fileName = System.getenv("APP_PROPS");
    if (fileName == null) {
      fileName = System.getProperty("app.props");
      if (fileName == null) {
        fileName = "application.properties";
      }
    }

    globalMap = PropertyMapLoader.load(null, fileName);
    if (globalMap == null) {
      globalMap = new PropertyMap();
    }
  }

  /**
   * Return the property map loading it if required.
   */
  private static synchronized PropertyMap getPropertyMap() {

    if (globalMap == null) {
      initPropertyMap();
    }

    return globalMap;
  }

  /**
   * Return a String property.
   */
  public static synchronized String get(String key) {
    return getPropertyMap().get(key, null);
  }

  /**
   * Return a String property with a default value.
   */
  public static synchronized String get(String key, String defaultValue) {
    return getPropertyMap().get(key, defaultValue);
  }

  /**
   * Return a int property with a default value.
   */
  public static synchronized int getInt(String key, int defaultValue) {
    return getPropertyMap().getInt(key, defaultValue);
  }

  /**
   * Return a boolean property with a default value.
   */
  public static synchronized boolean getBoolean(String key, boolean defaultValue) {
    return getPropertyMap().getBoolean(key, defaultValue);
  }

  /**
   * Set a property return the previous value. This will evaluate any
   * expressions in the value.
   */
  public static synchronized String put(String key, String value) {
    return getPropertyMap().putEval(key, value);
  }

  /**
   * Set a Map of key value properties.
   */
  public static synchronized void putAll(Map<String, String> keyValueMap) {
    for (Entry<String, String> e : keyValueMap.entrySet()) {
      getPropertyMap().putEval(e.getKey(), e.getValue());
    }
  }
}
